Package: ggbreak
Title: Set Axis Break for 'ggplot2'
Version: 0.1.7
Authors@R: c(
       person("Guangchuang", "Yu",     email = "guangchuangyu@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-6485-8781")),
       person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "aut", comment = c(ORCID="0000-0003-3513-5362"))
       )
Description: An implementation of scale functions for setting axis breaks of a 'gg' plot (S Xu (2021) <doi:10.3389/fgene.2021.774846>).
Imports: ggfun (>= 0.1.1), grid, ggplot2, ggplotify (>= 0.0.7), aplot
        (>= 0.1.5), rlang, stats, yulab.utils (>= 0.1.9)
Suggests: cowplot, ggimage, knitr, patchwork, pillar, prettydoc,
        rmarkdown, testthat
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
URL: https://github.com/YuLab-SMU/ggbreak
BugReports: https://github.com/YuLab-SMU/ggbreak/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-18 07:35:11 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [aut] (ORCID: <https://orcid.org/0000-0003-3513-5362>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-18 08:02:04 UTC
