# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,gg)
S3method(ggplot_add,ggbreak)
S3method(ggplot_add,ggbreak_params)
S3method(ggplot_add,ggcut)
S3method(ggplot_add,ggcut_params)
S3method(ggplot_add,ggwrap)
S3method(ggplot_add,wrap_params)
S3method(grid.draw,ggbreak)
S3method(grid.draw,ggcut)
S3method(grid.draw,ggwrap)
S3method(plot,ggbreak)
S3method(print,ggbreak)
S3method(print,ggcut)
S3method(print,ggwrap)
export(scale_wrap)
export(scale_x_break)
export(scale_x_cut)
export(scale_y_break)
export(scale_y_cut)
importFrom(aplot,plot_list)
importFrom(ggfun,ggbreak2ggplot)
importFrom(ggfun,is.ggbreak)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplotify,as.ggplot)
importFrom(grid,grid.draw)
importFrom(rlang,abort)
importFrom(stats,setNames)
importFrom(utils,packageDescription)
