% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggbreak-package.R
\docType{package}
\name{ggbreak-package}
\alias{ggbreak}
\alias{ggbreak-package}
\title{ggbreak: Set Axis Break for 'ggplot2'}
\description{
An implementation of scale functions for setting axis breaks of a 'gg' plot (S Xu (2021) \doi{10.3389/fgene.2021.774846}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/YuLab-SMU/ggbreak}
  \item Report bugs at \url{https://github.com/YuLab-SMU/ggbreak/issues}
}

}
\author{
\strong{Maintainer}: Guangchuang Yu \email{guangchuangyu@gmail.com} (\href{https://orcid.org/0000-0002-6485-8781}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Shuangbin Xu \email{xshuangbin@163.com} (\href{https://orcid.org/0000-0003-3513-5362}{ORCID})
}

}
\keyword{internal}
