% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_wrap}
\alias{scale_wrap}
\title{scale-wrap}
\usage{
scale_wrap(n)
}
\arguments{
\item{n}{the number of subplot pieces.}
}
\value{
gg object
}
\description{
This scale function wraps a 'gg' plot over multiple rows to make plots with long x axes easier to read.
}
\examples{
library(ggplot2)
library(ggbreak)
p <- ggplot(economics, aes(x=date, y = unemploy, colour = uempmed)) +
     geom_line()
p + scale_wrap(n=4)
}
